
function Poisoner_MenuButton_OnClick()
	local _, _, itemName = string.find(this:GetName(), "^PoisonerMenuButton(.*)$");
	local slot;
	if (arg1 == "RightButton") then
		slot = "SecondaryHandSlot";
	else
		slot = "MainHandSlot";
	end
	Poisoner_PickupPoison(itemName);
	PickupInventoryItem(GetInventorySlotInfo(slot));

	-- If the pickup failed we will now have the weapon on our cursor
	if (CursorHasItem()) then
		PickupInventoryItem(GetInventorySlotInfo(slot));
	end
	
	PoisonerMenu:Hide();
end


-- OnLoad event
function Poisoner_OnLoad()
	-- register for clicks
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	this:RegisterForDrag("LeftButton");
	-- need this for the initial load
	this:RegisterEvent("VARIABLES_LOADED");
	-- register a slash command
        SLASH_POISONER1 = "/poisoner";
        SlashCmdList["POISONER"] = function(msg)
                Poisoner_ProcessCommand(msg);
        end;
	
	PoisonerDebug("Poisoner Loaded");
end

function Poisoner_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		Poisoner_OnVariablesLoaded();
	end
end

function Poisoner_OnVariablesLoaded()
	if (POISONER_CONFIG == nil) then
		POISONER_CONFIG = {};
		local _, cls = UnitClass("player");
		POISONER_CONFIG.enabled = (cls == "ROGUE");
	end
	if (POISONER_CONFIG.enabled) then
		PoisonerMinimapButton:Show();
	end
	this:UnregisterEvent("VARIABLES_LOADED");
end




-- Show/Hide the poison menu
function Poisoner_MMButton_OnClick(button)
	if PoisonerMenu:IsShown() then
		PoisonerMenu:Hide();
	else
		Poisoner_CreateButtons();
		PoisonerMenu:Show();
	end
end


-- This function creates/initialises the individual buttons for each poison
-- type. Note that we re-use buttons if they had been previously created, as
-- there is no delete operation for frames.
function Poisoner_CreateButtons()
	local menu = PoisonerMenu;
	local i = 0;
	PoisonerDebug("Children of PoisonerMenu:")
	local children = { menu:GetChildren() };
	for _, child in ipairs(children) do
		PoisonerDebug(child:GetName());
		i = i+1;
	end
	PoisonerDebug(i.." children");

	local x = 0;
	for _, t in pairs(Poisoner_FindPoisons()) do
		local buttonName = "PoisonerMenuButton"..t.name;
		local button = getglobal(buttonName);
		if (button == nil) then
			button = CreateFrame("Button", buttonName, menu);
			button:SetFrameStrata("MEDIUM");
			button:SetWidth(25);
			button:SetHeight(25);
			local texture = button:CreateTexture(nil, "BACKGROUND");
			texture:SetTexture(t.icon);
			texture:SetAllPoints(button);
			button.texture = texture;
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
			button:SetScript("OnEnter", loadstring("Poisoner_ShowTooltip(\""..t.name.."\");"));
			button:SetScript("OnLeave", function() GameTooltip:Hide() end);
			button:SetScript("OnClick", Poisoner_MenuButton_OnClick);
			button.count = button:CreateFontString(buttonName.."Text");
			button.count:SetFontObject("GameFontHighlightSmallOutline");
			button.count:SetPoint("BOTTOMLEFT", 0, 0);
		end
		button.count:SetText(""..t.quantity);
		button:SetPoint("TOPRIGHT", x, 0);
		x = x - 25;
		button:Show();
	end
end

--
-- Find available poisons
-- Returns an table keyed by poison name, value a table containing texture and quantity
--
function Poisoner_FindPoisons()
	local i, itemLink, bagSlots, bagId, icon, quantity;
	local ret = {};
	for i=0, 4 do
		itemLink = nil;
		bagSlots = GetContainerNumSlots(i);
		if (bagSlots > 0) then
			for j=1, bagSlots do
				itemLink = nil;
				itemLink = GetContainerItemLink(i, j);
				icon, quantity = GetContainerItemInfo(i, j)
				if (itemLink) then
					local itemName = nil;
					local idx,pattern;
					for idx, pattern in ipairs(Poisoner_Patterns) do
						_, _, itemName = string.find(itemLink, "%[("..pattern..")%]");
						if (itemName ~= nil) then
							break;
						end
					end
					
					if (itemName ~= nil) then
						-- if the quantity is 1, we either have a single consumable item, or an expendable item
						if (quantity == 1) then
							local charges = Poisoner_CountCharges(i, j);
							if (charges ~= nil) then
								quantity = charges;
							end
						end

						local t = ret[itemName];
						if (t == nil) then
							t = { name=itemName, icon=icon, quantity=0 };
							ret[itemName] = t;
						end
						t.quantity = t.quantity + quantity;
					end
				end
			end
		end
	end
	return ret;
end

-- Use a tooltip to scan for the number of 'charges' on expendable items.
-- Returns the number of charges, or nil if it could not be found.
function Poisoner_CountCharges(bagId, bagSlot)
PoisonerDebug("Looking for charges of "..bagId..","..bagSlot);
	local ret = nil;
	PoisonerScannerTooltip:SetOwner(PoisonerScannerTooltip, "ANCHOR_NONE");
	PoisonerScannerTooltip:SetBagItem(bagId, bagSlot);
	for i=1,PoisonerScannerTooltip:NumLines() do
		local field = getglobal("PoisonerScannerTooltipTextLeft"..i);
		if (field ~= nil) then
			local text = field:GetText();
			if (text) then
				local _, _, charges = string.find(text, POISONER_PATTERN_CHARGES);
				if (charges) then
					ret = tonumber(charges);
					break;
				end
			end
		end
	end
	PoisonerScannerTooltip:Hide();
	return ret;
end

-- Activate the named poison
function Poisoner_PickupPoison(name)
	local i, itemLink, bagSlots, bagId, icon, quantity;
	for i=0, 4 do
		itemLink = nil;
		bagSlots = GetContainerNumSlots(i);
		if (bagSlots > 0) then
			for j=1, bagSlots do
				itemLink = nil;
				itemLink = GetContainerItemLink(i, j);
				if (itemLink) then
					local _, _, itemName = string.find(itemLink, "%[(.*)%]");
					if (itemName ~= nil and itemName == name) then
PoisonerDebug("Found "..name);
						UseContainerItem(i, j);
						return;
					end
				end
			end
		end
	end
end


--
-- Slash command processing
--
function Poisoner_ProcessCommand(command)
	if (command == "lock") then
		Poisoner_DragLock();
	elseif (command == "unlock") then
		Poisoner_DragUnlock();
	elseif (command == "enable") then
		Poisoner_Enable();
	elseif (command == "disable") then
		Poisoner_Disable();
	else
		ChatFrame1:AddMessage(POISONER_COMMANDERROR..command, 1.0, 1.0, 0.5);
	end
end

function Poisoner_Disable()
	POISONER_CONFIG.enabled = false;
	if (PoisonerMenu:IsShown()) then
		PoisonerMenu:Hide();
	end
	PoisonerMinimapButton:Hide();
end

function Poisoner_Enable()
	POISONER_CONFIG.enabled = true;
	PoisonerMinimapButton:Show();
end

--
-- Dragging
-- This is probably not working yet
--

-- Start dragging the poison button
function Poisoner_OnDragStart()
	if (not POISONER_CONFIG.dragLock and not Poisoner_BeingDragged) then
		this:StartMoving();
		Poisoner_BeingDragged = true;
	else
		ChatFrame1:AddMessage(POISONER_DRAGERROR, 1.0, 1.0, 0.5);
	end
end

-- Stop dragging the shard button
function Poisoner_OnDragStop()
	if (Poisoner_BeingDragged) then
		this:StopMovingOrSizing()
		Poisoner_BeingDragged = false;
	end
end

function Poisoner_DragLock()
	POISONER_CONFIG.dragLock = true;
end

function Poisoner_DragUnlock()
	POISONER_CONFIG.dragLock = false;
end



--
-- Tooltips
--

-- Show the tooltip text
function Poisoner_ShowTooltip(text)

	-- Set the anchor and text for the tooltip.
	easyclear("GameTooltip");
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT");
	GameTooltip:SetText(text, 1.00, 1.00, 1.00);
    
	GameTooltip:Show();
end


function easyclear( tooltipBase )
	for i=1, 15, 1 do
		getglobal(tooltipBase.."TextLeft"..i):SetText("");
		getglobal(tooltipBase.."TextRight"..i):SetText("");
	end		
end


--
-- Debugging
--

function PoisonerDebug(message)
	if (false) then
		if (message == nil) then
			message = "<nil>";
		end
		ChatFrame1:AddMessage(message);
	end
end

